/* Agar icon reference file */
/* Generated by agarpaint 1.1 */

#include <agar/gui/iconmgr.h>
#include <agar/gui/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC AG_StaticIcon agIconLeftButton;
extern DECLSPEC AG_StaticIcon agIconMidButton;
extern DECLSPEC AG_StaticIcon agIconRightButton;
extern DECLSPEC AG_StaticIcon agIconCtrlKey;
extern DECLSPEC AG_StaticIcon agIconLoad;
extern DECLSPEC AG_StaticIcon agIconSave;
extern DECLSPEC AG_StaticIcon agIconUp;
extern DECLSPEC AG_StaticIcon agIconDown;
extern DECLSPEC AG_StaticIcon agIconTrash;
extern DECLSPEC AG_StaticIcon agIconClose;
extern DECLSPEC AG_StaticIcon agIconDoc;
extern DECLSPEC AG_StaticIcon agIconSymLink;
extern DECLSPEC AG_StaticIcon agIconDirectory;
extern DECLSPEC AG_StaticIcon agIconSmallArrowRight;
extern DECLSPEC AG_StaticIcon agIconWinClose;
extern DECLSPEC AG_StaticIcon agIconWinMinimize;
extern DECLSPEC AG_StaticIcon agIconWinMaximize;
extern DECLSPEC AG_StaticIcon agIconMagnifier;
extern DECLSPEC AG_StaticIcon agIconGear;
extern DECLSPEC AG_StaticIcon agIconDocImport;
extern DECLSPEC AG_StaticIcon agIconWindow;
__END_DECLS
/* Close generated block */
#include <agar/gui/close.h>
