/*	Public domain	*/

#ifndef _AGAR_MATH_M_H_
#define _AGAR_MATH_M_H_

#include <agar/math/begin.h>

#include <agar/math/m_math.h>
#include <agar/math/m_vectorz.h>
#include <agar/math/m_complex.h>
#include <agar/math/m_vector.h>
#include <agar/math/m_matrix.h>
#include <agar/math/m_quaternion.h>
#include <agar/math/m_coordinates.h>
#include <agar/math/m_color.h>
#include <agar/math/m_geometry.h>

#include <agar/math/close.h>

#endif /* _AGAR_MATH_M_H_ */
