/*	Public domain	*/

#ifndef _AGAR_SG_PUBLIC_H_
#define _AGAR_SG_PUBLIC_H_

#include <agar/core.h>
#include <agar/gui.h>

#include <agar/math/m.h>
#include <agar/math/m_gui.h>

#include <agar/sg/sg.h>
#include <agar/sg/sg_gui.h>

#endif /* _AGAR_SG_PUBLIC_H_ */
