/* Agar icon reference file */
/* Generated by agarpaint 1.1 */

#include <agar/map/begin.h>
/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC AG_StaticIcon rgIconStamp;
extern DECLSPEC AG_StaticIcon rgIconMagnifier;
extern DECLSPEC AG_StaticIcon rgIconLayerEditor;
extern DECLSPEC AG_StaticIcon rgIconMirror;
extern DECLSPEC AG_StaticIcon rgIconFlip;
extern DECLSPEC AG_StaticIcon rgIconInvert;
extern DECLSPEC AG_StaticIcon rgIconHGrad;
extern DECLSPEC AG_StaticIcon rgIconVGrad;
extern DECLSPEC AG_StaticIcon rgIconCGrad;
extern DECLSPEC AG_StaticIcon rgIconTiling;
extern DECLSPEC AG_StaticIcon rgIconSketchProj;
extern DECLSPEC AG_StaticIcon rgIconSketch;
extern DECLSPEC AG_StaticIcon rgIconSketchAttach;
extern DECLSPEC AG_StaticIcon rgIconPixmapResize;
extern DECLSPEC AG_StaticIcon rgIconPixmapAttach;
extern DECLSPEC AG_StaticIcon rgIconPolygon;
extern DECLSPEC AG_StaticIcon rgIconExtrusion;
extern DECLSPEC AG_StaticIcon rgIconRevolution;
extern DECLSPEC AG_StaticIcon rgIconSwap;
extern DECLSPEC AG_StaticIcon rgIconControls;
extern DECLSPEC AG_StaticIcon rgIconPixmap;
extern DECLSPEC AG_StaticIcon rgIconWalkable;
extern DECLSPEC AG_StaticIcon rgIconClimbable;
extern DECLSPEC AG_StaticIcon rgIconJumpable;
extern DECLSPEC AG_StaticIcon rgIconSlippery;
extern DECLSPEC AG_StaticIcon rgIconObject;
extern DECLSPEC AG_StaticIcon rgIconMap_n0;
extern DECLSPEC AG_StaticIcon rgIconDrawing;
extern DECLSPEC AG_StaticIcon rgIconTileset;
extern DECLSPEC AG_StaticIcon rgIconEdit;
extern DECLSPEC AG_StaticIcon rgIconDuplicate;
extern DECLSPEC AG_StaticIcon rgIconOrigin;
extern DECLSPEC AG_StaticIcon rgIconGrid;
extern DECLSPEC AG_StaticIcon rgIconPlay;
extern DECLSPEC AG_StaticIcon rgIconPause;
extern DECLSPEC AG_StaticIcon rgIconStop;
extern DECLSPEC AG_StaticIcon rgIconRewind;
extern DECLSPEC AG_StaticIcon rgIconForward;
extern DECLSPEC AG_StaticIcon rgIconRewindFrame;
extern DECLSPEC AG_StaticIcon rgIconForwardFrame;
extern DECLSPEC AG_StaticIcon rgIconFill;
__END_DECLS
/* Close generated block */
#include <agar/map/close.h>
