/*	Public domain	*/

#ifndef _AGAR_MATH_M_BEZIER_PRIMITIVES_H
#define _AGAR_MATH_M_BEZIER_PRIMITIVES_H

#include <agar/math/begin.h>

/* Begin generated block */
__BEGIN_DECLS
extern DECLSPEC void M_DrawBezier2(AG_Widget*, M_Real, M_Real, M_Real, M_Real, M_Real, M_Real, M_Real, M_Real, int, AG_Color*);
__END_DECLS
/* Close generated block */

#include <agar/math/close.h>

#endif /* _AGAR_MATH_M_BEZIER_PRIMITIVES_H */
